// This is a C header file to be used by the output of the Cyclone
// to C translator.  The corresponding definitions are in file lib/runtime_cyc.c
#ifndef _CYC_INCLUDE_H_
#define _CYC_INCLUDE_H_

#include <setjmp.h>

#ifdef NO_CYC_PREFIX
#define ADD_PREFIX(x) x
#else
#define ADD_PREFIX(x) Cyc_##x
#endif

#ifndef offsetof
// should be size_t, but int is fine.
#define offsetof(t,n) ((int)(&(((t *)0)->n)))
#endif

//// Tagged arrays
struct _tagged_arr { 
  unsigned char *curr; 
  unsigned char *base; 
  unsigned char *last_plus_one; 
};

//// Discriminated Unions
struct _xtunion_struct { char *tag; };

// Need one of these per thread (we don't have threads)
// The runtime maintains a stack that contains either _handler_cons
// structs or _RegionHandle structs.  The tag is 0 for a handler_cons
// and 1 for a region handle.  
struct _RuntimeStack {
  int tag; // 0 for an exception handler, 1 for a region handle
  struct _RuntimeStack *next;
};

//// Regions
struct _RegionPage {
#ifdef CYC_REGION_PROFILE
  unsigned total_bytes;
  unsigned free_bytes;
#endif
  struct _RegionPage *next;
  char data[0];
};

struct _RegionHandle {
  struct _RuntimeStack s;
  struct _RegionPage *curr;
  char               *offset;
  char               *last_plus_one;
#ifdef CYC_REGION_PROFILE
  const char         *name;
#endif
};

extern struct _RegionHandle _new_region(const char *);
extern void * _region_malloc(struct _RegionHandle *, unsigned);
extern void * _region_calloc(struct _RegionHandle *, unsigned t, unsigned n);
extern void   _free_region(struct _RegionHandle *);
extern void   _reset_region(struct _RegionHandle *);

//// Exceptions 
struct _handler_cons {
  struct _RuntimeStack s;
  jmp_buf handler;
};
extern void _push_handler(struct _handler_cons *);
extern void _push_region(struct _RegionHandle *);
extern void _npop_handler(int);
extern void _pop_handler();
extern void _pop_region();

#ifndef _throw
extern int _throw_null();
extern int _throw_arraybounds();
extern int _throw_badalloc();
extern int _throw(void* e);
#endif

extern struct _xtunion_struct *_exn_thrown;

//// Built-in Exceptions
extern struct _xtunion_struct ADD_PREFIX(Null_Exception_struct);
extern struct _xtunion_struct * ADD_PREFIX(Null_Exception);
extern struct _xtunion_struct ADD_PREFIX(Array_bounds_struct);
extern struct _xtunion_struct * ADD_PREFIX(Array_bounds);
extern struct _xtunion_struct ADD_PREFIX(Match_Exception_struct);
extern struct _xtunion_struct * ADD_PREFIX(Match_Exception);
extern struct _xtunion_struct ADD_PREFIX(Bad_alloc_struct);
extern struct _xtunion_struct * ADD_PREFIX(Bad_alloc);

//// Built-in Run-time Checks and company
#ifdef __APPLE__
#define _INLINE_FUNCTIONS
#endif

#ifdef NO_CYC_NULL_CHECKS
#define _check_null(ptr) (ptr)
#else
#ifdef _INLINE_FUNCTIONS
static inline void *
_check_null(void *ptr) {
  void*_check_null_temp = (void*)(ptr);
  if (!_check_null_temp) _throw_null();
  return _check_null_temp;
}
#else
#define _check_null(ptr) \
  ({ void*_check_null_temp = (void*)(ptr); \
     if (!_check_null_temp) _throw_null(); \
     _check_null_temp; })
#endif
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  ((char *)ptr) + (elt_sz)*(index); })
#else
#ifdef _INLINE_FUNCTIONS
static inline char *
_check_known_subscript_null(void *ptr, unsigned bound, unsigned elt_sz, unsigned index) {
  void*_cks_ptr = (void*)(ptr);
  unsigned _cks_bound = (bound);
  unsigned _cks_elt_sz = (elt_sz);
  unsigned _cks_index = (index);
  if (!_cks_ptr) _throw_null();
  if (_cks_index >= _cks_bound) _throw_arraybounds();
  return ((char *)_cks_ptr) + _cks_elt_sz*_cks_index;
}
#else
#define _check_known_subscript_null(ptr,bound,elt_sz,index) ({ \
  void*_cks_ptr = (void*)(ptr); \
  unsigned _cks_bound = (bound); \
  unsigned _cks_elt_sz = (elt_sz); \
  unsigned _cks_index = (index); \
  if (!_cks_ptr) _throw_null(); \
  if (_cks_index >= _cks_bound) _throw_arraybounds(); \
  ((char *)_cks_ptr) + _cks_elt_sz*_cks_index; })
#endif
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _check_known_subscript_notnull(bound,index) (index)
#else
#ifdef _INLINE_FUNCTIONS
static inline unsigned
_check_known_subscript_notnull(unsigned bound,unsigned index) { 
  unsigned _cksnn_bound = (bound); 
  unsigned _cksnn_index = (index); 
  if (_cksnn_index >= _cksnn_bound) _throw_arraybounds(); 
  return _cksnn_index;
}
#else
#define _check_known_subscript_notnull(bound,index) ({ \
  unsigned _cksnn_bound = (bound); \
  unsigned _cksnn_index = (index); \
  if (_cksnn_index >= _cksnn_bound) _throw_arraybounds(); \
  _cksnn_index; })
#endif
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#ifdef _INLINE_FUNCTIONS
static inline unsigned char *
_check_unknown_subscript(struct _tagged_arr arr,unsigned elt_sz,unsigned index) {
  struct _tagged_arr _cus_arr = (arr);
  unsigned _cus_elt_sz = (elt_sz);
  unsigned _cus_index = (index);
  unsigned char *_cus_ans = _cus_arr.curr + _cus_elt_sz * _cus_index;
  return _cus_ans;
}
#else
#define _check_unknown_subscript(arr,elt_sz,index) ({ \
  struct _tagged_arr _cus_arr = (arr); \
  unsigned _cus_elt_sz = (elt_sz); \
  unsigned _cus_index = (index); \
  unsigned char *_cus_ans = _cus_arr.curr + _cus_elt_sz * _cus_index; \
  _cus_ans; })
#endif
#else
#ifdef _INLINE_FUNCTIONS
static inline unsigned char *
_check_unknown_subscript(struct _tagged_arr arr,unsigned elt_sz,unsigned index) {
  struct _tagged_arr _cus_arr = (arr);
  unsigned _cus_elt_sz = (elt_sz);
  unsigned _cus_index = (index);
  unsigned char *_cus_ans = _cus_arr.curr + _cus_elt_sz * _cus_index;
  if (!_cus_arr.base) _throw_null();
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one)
    _throw_arraybounds();
  return _cus_ans;
}
#else
#define _check_unknown_subscript(arr,elt_sz,index) ({ \
  struct _tagged_arr _cus_arr = (arr); \
  unsigned _cus_elt_sz = (elt_sz); \
  unsigned _cus_index = (index); \
  unsigned char *_cus_ans = _cus_arr.curr + _cus_elt_sz * _cus_index; \
  if (!_cus_arr.base) _throw_null(); \
  if (_cus_ans < _cus_arr.base || _cus_ans >= _cus_arr.last_plus_one) \
    _throw_arraybounds(); \
  _cus_ans; })
#endif
#endif

#ifdef _INLINE_FUNCTIONS
static inline struct _tagged_arr
_tag_arr(const void *tcurr,unsigned elt_sz,unsigned num_elts) {
  struct _tagged_arr _tag_arr_ans;
  _tag_arr_ans.base = _tag_arr_ans.curr = (void*)(tcurr);
  _tag_arr_ans.last_plus_one = _tag_arr_ans.base + (elt_sz) * (num_elts);
  return _tag_arr_ans;
}
#else
#define _tag_arr(tcurr,elt_sz,num_elts) ({ \
  struct _tagged_arr _tag_arr_ans; \
  _tag_arr_ans.base = _tag_arr_ans.curr = (void*)(tcurr); \
  _tag_arr_ans.last_plus_one = _tag_arr_ans.base + (elt_sz) * (num_elts); \
  _tag_arr_ans; })
#endif

#ifdef _INLINE_FUNCTIONS
static inline struct _tagged_arr *
_init_tag_arr(struct _tagged_arr *arr_ptr,
              void *arr, unsigned elt_sz, unsigned num_elts) {
  struct _tagged_arr *_itarr_ptr = (arr_ptr);
  void* _itarr = (arr);
  _itarr_ptr->base = _itarr_ptr->curr = _itarr;
  _itarr_ptr->last_plus_one = ((char *)_itarr) + (elt_sz) * (num_elts);
  return _itarr_ptr;
}
#else
#define _init_tag_arr(arr_ptr,arr,elt_sz,num_elts) ({ \
  struct _tagged_arr *_itarr_ptr = (arr_ptr); \
  void* _itarr = (arr); \
  _itarr_ptr->base = _itarr_ptr->curr = _itarr; \
  _itarr_ptr->last_plus_one = ((char *)_itarr) + (elt_sz) * (num_elts); \
  _itarr_ptr; })
#endif

#ifdef NO_CYC_BOUNDS_CHECKS
#define _untag_arr(arr,elt_sz,num_elts) ((arr).curr)
#else
#ifdef _INLINE_FUNCTIONS
static inline unsigned char *
_untag_arr(struct _tagged_arr arr, unsigned elt_sz,unsigned num_elts) {
  struct _tagged_arr _arr = (arr);
  unsigned char *_curr = _arr.curr;
  if (_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one)
    _throw_arraybounds();
  return _curr;
}
#else
#define _untag_arr(arr,elt_sz,num_elts) ({ \
  struct _tagged_arr _arr = (arr); \
  unsigned char *_curr = _arr.curr; \
  if (_curr < _arr.base || _curr + (elt_sz) * (num_elts) > _arr.last_plus_one)\
    _throw_arraybounds(); \
  _curr; })
#endif
#endif

#ifdef _INLINE_FUNCTIONS
static inline unsigned
_get_arr_size(struct _tagged_arr arr,unsigned elt_sz) {
  struct _tagged_arr _get_arr_size_temp = (arr);
  unsigned char *_get_arr_size_curr=_get_arr_size_temp.curr;
  unsigned char *_get_arr_size_last=_get_arr_size_temp.last_plus_one;
  return (_get_arr_size_curr < _get_arr_size_temp.base ||
          _get_arr_size_curr >= _get_arr_size_last) ? 0 :
    ((_get_arr_size_last - _get_arr_size_curr) / (elt_sz));
}
#else
#define _get_arr_size(arr,elt_sz) \
  ({struct _tagged_arr _get_arr_size_temp = (arr); \
    unsigned char *_get_arr_size_curr=_get_arr_size_temp.curr; \
    unsigned char *_get_arr_size_last=_get_arr_size_temp.last_plus_one; \
    (_get_arr_size_curr < _get_arr_size_temp.base || \
     _get_arr_size_curr >= _get_arr_size_last) ? 0 : \
    ((_get_arr_size_last - _get_arr_size_curr) / (elt_sz));})
#endif

#ifdef _INLINE_FUNCTIONS
static inline struct _tagged_arr
_tagged_arr_plus(struct _tagged_arr arr,unsigned elt_sz,int change) {
  struct _tagged_arr _ans = (arr);
  _ans.curr += ((int)(elt_sz))*(change);
  return _ans;
}
#else
#define _tagged_arr_plus(arr,elt_sz,change) ({ \
  struct _tagged_arr _ans = (arr); \
  _ans.curr += ((int)(elt_sz))*(change); \
  _ans; })
#endif

#ifdef _INLINE_FUNCTIONS
static inline struct _tagged_arr
_tagged_arr_inplace_plus(struct _tagged_arr *arr_ptr,unsigned elt_sz,int change) {
  struct _tagged_arr * _arr_ptr = (arr_ptr);
  _arr_ptr->curr += ((int)(elt_sz))*(change);
  return *_arr_ptr;
}
#else
#define _tagged_arr_inplace_plus(arr_ptr,elt_sz,change) ({ \
  struct _tagged_arr * _arr_ptr = (arr_ptr); \
  _arr_ptr->curr += ((int)(elt_sz))*(change); \
  *_arr_ptr; })
#endif

#ifdef _INLINE_FUNCTIONS
static inline struct _tagged_arr
_tagged_arr_inplace_plus_post(struct _tagged_arr *arr_ptr,unsigned elt_sz,int change) {
  struct _tagged_arr * _arr_ptr = (arr_ptr);
  struct _tagged_arr _ans = *_arr_ptr;
  _arr_ptr->curr += ((int)(elt_sz))*(change);
  return _ans;
}
#else
#define _tagged_arr_inplace_plus_post(arr_ptr,elt_sz,change) ({ \
  struct _tagged_arr * _arr_ptr = (arr_ptr); \
  struct _tagged_arr _ans = *_arr_ptr; \
  _arr_ptr->curr += ((int)(elt_sz))*(change); \
  _ans; })
#endif

// Decrease the upper bound on a fat pointer by numelts where sz is
// the size of the pointer's type.  Note that this can't be a macro
// if we're to get initializers right.
static struct _tagged_arr _tagged_ptr_decrease_size(struct _tagged_arr x,
                                                    unsigned int sz,
                                                    unsigned int numelts) {
  x.last_plus_one -= sz * numelts; 
  return x; 
}

// Add i to zero-terminated pointer x.  Checks for x being null and
// ensures that x[0..i-1] are not 0.
#ifdef NO_CYC_BOUNDS_CHECK
#define _zero_arr_plus(orig_x,orig_sz,orig_i) ((orig_x)+(orig_i))
#else
#define _zero_arr_plus(orig_x,orig_sz,orig_i) ({ \
  typedef _czs_tx = (*orig_x); \
  _czs_tx *_czs_x = (_czs_tx *)(orig_x); \
  unsigned int _czs_sz = (orig_sz); \
  int _czs_i = (orig_i); \
  unsigned int _czs_temp; \
  if ((_czs_x) == 0) _throw_null(); \
  if (_czs_i < 0) _throw_arraybounds(); \
  for (_czs_temp=_czs_sz; _czs_temp < _czs_i; _czs_temp++) \
    if (_czs_x[_czs_temp] == 0) _throw_arraybounds(); \
  _czs_x+_czs_i; })
#endif

// Calculates the number of elements in a zero-terminated, thin array.
// If non-null, the array is guaranteed to have orig_offset elements.
#define _get_zero_arr_size(orig_x,orig_offset) ({ \
  typedef _gres_tx = (*orig_x); \
  _gres_tx *_gres_x = (_gres_tx *)(orig_x); \
  unsigned int _gres_offset = (orig_offset); \
  unsigned int _gres = 0; \
  if (_gres_x != 0) { \
     _gres = _gres_offset; \
     _gres_x += _gres_offset - 1; \
     while (*_gres_x != 0) { _gres_x++; _gres++; } \
  } _gres; })

// Does in-place addition of a zero-terminated pointer (x += e and ++x).  
// Note that this expands to call _zero_arr_plus.
#define _zero_arr_inplace_plus(x,orig_i) ({ \
  typedef _zap_tx = (*x); \
  _zap_tx **_zap_x = &((_zap_tx*)x); \
  *_zap_x = _zero_arr_plus(*_zap_x,1,(orig_i)); })

// Does in-place increment of a zero-terminated pointer (e.g., x++).
// Note that this expands to call _zero_arr_plus.
#define _zero_arr_inplace_plus_post(x,orig_i) ({ \
  typedef _zap_tx = (*x); \
  _zap_tx **_zap_x = &((_zap_tx*)x); \
  _zap_tx *_zap_res = *_zap_x; \
  *_zap_x = _zero_arr_plus(_zap_res,1,(orig_i)); \
  _zap_res; })
  
//// Allocation
extern void* GC_malloc(int);
extern void* GC_malloc_atomic(int);
extern void* GC_calloc(unsigned,unsigned);
extern void* GC_calloc_atomic(unsigned,unsigned);

static inline void* _cycalloc(int n) {
  void * ans = (void *)GC_malloc(n);
  if(!ans)
    _throw_badalloc();
  return ans;
}
static inline void* _cycalloc_atomic(int n) {
  void * ans = (void *)GC_malloc_atomic(n);
  if(!ans)
    _throw_badalloc();
  return ans;
}
static inline void* _cyccalloc(unsigned n, unsigned s) {
  void* ans = (void*)GC_calloc(n,s);
  if (!ans)
    _throw_badalloc();
  return ans;
}
static inline void* _cyccalloc_atomic(unsigned n, unsigned s) {
  void* ans = (void*)GC_calloc_atomic(n,s);
  if (!ans)
    _throw_badalloc();
  return ans;
}
#define MAX_MALLOC_SIZE (1 << 28)
static inline unsigned int _check_times(unsigned x, unsigned y) {
  unsigned long long whole_ans = 
    ((unsigned long long)x)*((unsigned long long)y);
  unsigned word_ans = (unsigned)whole_ans;
  if(word_ans < whole_ans || word_ans > MAX_MALLOC_SIZE)
    _throw_badalloc();
  return word_ans;
}

#if defined(CYC_REGION_PROFILE) 
extern void* _profile_GC_malloc(int,char *file,int lineno);
extern void* _profile_GC_malloc_atomic(int,char *file,int lineno);
extern void* _profile_region_malloc(struct _RegionHandle *, unsigned,
                                     char *file,int lineno);
extern struct _RegionHandle _profile_new_region(const char *rgn_name,
						char *file,int lineno);
extern void _profile_free_region(struct _RegionHandle *,
				 char *file,int lineno);
#  if !defined(RUNTIME_CYC)
#define _new_region(n) _profile_new_region(n,__FILE__ ":" __FUNCTION__,__LINE__)
#define _free_region(r) _profile_free_region(r,__FILE__ ":" __FUNCTION__,__LINE__)
#define _region_malloc(rh,n) _profile_region_malloc(rh,n,__FILE__ ":" __FUNCTION__,__LINE__)
#  endif
#define _cycalloc(n) _profile_GC_malloc(n,__FILE__ ":" __FUNCTION__,__LINE__)
#define _cycalloc_atomic(n) _profile_GC_malloc_atomic(n,__FILE__ ":" __FUNCTION__,__LINE__)
#endif
#endif
 void exit(int);void*abort();struct Cyc_Core_Opt{void*v;};extern char Cyc_Core_Invalid_argument[
21];struct Cyc_Core_Invalid_argument_struct{char*tag;struct _tagged_arr f1;};extern
char Cyc_Core_Failure[12];struct Cyc_Core_Failure_struct{char*tag;struct
_tagged_arr f1;};extern char Cyc_Core_Impossible[15];struct Cyc_Core_Impossible_struct{
char*tag;struct _tagged_arr f1;};extern char Cyc_Core_Not_found[14];extern char Cyc_Core_Unreachable[
16];struct Cyc_Core_Unreachable_struct{char*tag;struct _tagged_arr f1;};extern
struct _RegionHandle*Cyc_Core_heap_region;typedef struct{int __count;union{
unsigned int __wch;char __wchb[4];}__value;}Cyc___mbstate_t;typedef struct{int __pos;
Cyc___mbstate_t __state;}Cyc__G_fpos_t;typedef Cyc__G_fpos_t Cyc_fpos_t;struct Cyc___cycFILE;
extern struct Cyc___cycFILE*Cyc_stdout;struct Cyc_Cstdio___abstractFILE;struct Cyc_String_pa_struct{
int tag;struct _tagged_arr f1;};struct Cyc_Int_pa_struct{int tag;unsigned int f1;};
struct Cyc_Double_pa_struct{int tag;double f1;};struct Cyc_LongDouble_pa_struct{int
tag;long double f1;};struct Cyc_ShortPtr_pa_struct{int tag;short*f1;};struct Cyc_IntPtr_pa_struct{
int tag;unsigned int*f1;};struct _tagged_arr Cyc_aprintf(struct _tagged_arr,struct
_tagged_arr);int Cyc_fprintf(struct Cyc___cycFILE*,struct _tagged_arr,struct
_tagged_arr);struct Cyc_ShortPtr_sa_struct{int tag;short*f1;};struct Cyc_UShortPtr_sa_struct{
int tag;unsigned short*f1;};struct Cyc_IntPtr_sa_struct{int tag;int*f1;};struct Cyc_UIntPtr_sa_struct{
int tag;unsigned int*f1;};struct Cyc_StringPtr_sa_struct{int tag;struct _tagged_arr
f1;};struct Cyc_DoublePtr_sa_struct{int tag;double*f1;};struct Cyc_FloatPtr_sa_struct{
int tag;float*f1;};struct Cyc_CharPtr_sa_struct{int tag;struct _tagged_arr f1;};int
Cyc_printf(struct _tagged_arr,struct _tagged_arr);int Cyc_putc(int,struct Cyc___cycFILE*);
struct _tagged_arr Cyc_rprintf(struct _RegionHandle*,struct _tagged_arr,struct
_tagged_arr);int Cyc_snprintf(struct _tagged_arr,unsigned int,struct _tagged_arr,
struct _tagged_arr);int Cyc_sprintf(struct _tagged_arr,struct _tagged_arr,struct
_tagged_arr);int Cyc_vfprintf(struct Cyc___cycFILE*,struct _tagged_arr,struct
_tagged_arr);int Cyc_vprintf(struct _tagged_arr,struct _tagged_arr);struct
_tagged_arr Cyc_vrprintf(struct _RegionHandle*,struct _tagged_arr,struct _tagged_arr);
int Cyc_vsnprintf(struct _tagged_arr,unsigned int,struct _tagged_arr,struct
_tagged_arr);int Cyc_vsprintf(struct _tagged_arr,struct _tagged_arr,struct
_tagged_arr);extern char Cyc_FileCloseError[19];extern char Cyc_FileOpenError[18];
struct Cyc_FileOpenError_struct{char*tag;struct _tagged_arr f1;};struct Cyc_List_List{
void*hd;struct Cyc_List_List*tl;};extern char Cyc_List_List_mismatch[18];extern char
Cyc_List_Nth[8];typedef struct{int quot;int rem;}Cyc_div_t;typedef struct{int quot;
int rem;}Cyc_ldiv_t;void*abort();void exit(int);static struct _tagged_arr Cyc_parg2string(
void*x){void*_tmp0=x;_LL1: if(*((int*)_tmp0)!= 0)goto _LL3;_LL2: return({const char*
_tmp1="string";_tag_arr(_tmp1,sizeof(char),_get_zero_arr_size(_tmp1,7));});_LL3:
if(*((int*)_tmp0)!= 1)goto _LL5;_LL4: return({const char*_tmp2="int";_tag_arr(_tmp2,
sizeof(char),_get_zero_arr_size(_tmp2,4));});_LL5: if(*((int*)_tmp0)!= 2)goto _LL7;
_LL6: return({const char*_tmp3="double";_tag_arr(_tmp3,sizeof(char),
_get_zero_arr_size(_tmp3,7));});_LL7: if(*((int*)_tmp0)!= 3)goto _LL9;_LL8: return({
const char*_tmp4="long double";_tag_arr(_tmp4,sizeof(char),_get_zero_arr_size(
_tmp4,12));});_LL9: if(*((int*)_tmp0)!= 4)goto _LLB;_LLA: return({const char*_tmp5="short *";
_tag_arr(_tmp5,sizeof(char),_get_zero_arr_size(_tmp5,8));});_LLB: if(*((int*)
_tmp0)!= 5)goto _LL0;_LLC: return({const char*_tmp6="unsigned long *";_tag_arr(
_tmp6,sizeof(char),_get_zero_arr_size(_tmp6,16));});_LL0:;}static void*Cyc_badarg(
struct _tagged_arr s){return(void*)_throw((void*)({struct Cyc_Core_Invalid_argument_struct*
_tmp7=_cycalloc(sizeof(*_tmp7));_tmp7[0]=({struct Cyc_Core_Invalid_argument_struct
_tmp8;_tmp8.tag=Cyc_Core_Invalid_argument;_tmp8.f1=s;_tmp8;});_tmp7;}));}static
int Cyc_va_arg_int(struct _tagged_arr ap){void*_tmp9=*((void**)
_check_unknown_subscript(ap,sizeof(void*),0));unsigned int _tmpA;_LLE: if(*((int*)
_tmp9)!= 1)goto _LL10;_tmpA=((struct Cyc_Int_pa_struct*)_tmp9)->f1;_LLF: return(int)
_tmpA;_LL10:;_LL11: return((int(*)(struct _tagged_arr s))Cyc_badarg)(({const char*
_tmpB="printf expected int";_tag_arr(_tmpB,sizeof(char),_get_zero_arr_size(_tmpB,
20));}));_LLD:;}static int Cyc_va_arg_long(struct _tagged_arr ap){void*_tmpC=*((void**)
_check_unknown_subscript(ap,sizeof(void*),0));unsigned int _tmpD;_LL13: if(*((int*)
_tmpC)!= 1)goto _LL15;_tmpD=((struct Cyc_Int_pa_struct*)_tmpC)->f1;_LL14: return(
int)_tmpD;_LL15:;_LL16: return((int(*)(struct _tagged_arr s))Cyc_badarg)(({const
char*_tmpE="printf expected int";_tag_arr(_tmpE,sizeof(char),_get_zero_arr_size(
_tmpE,20));}));_LL12:;}static unsigned int Cyc_va_arg_ulong(struct _tagged_arr ap){
void*_tmpF=*((void**)_check_unknown_subscript(ap,sizeof(void*),0));unsigned int
_tmp10;_LL18: if(*((int*)_tmpF)!= 1)goto _LL1A;_tmp10=((struct Cyc_Int_pa_struct*)
_tmpF)->f1;_LL19: return(unsigned int)_tmp10;_LL1A:;_LL1B: return((unsigned int(*)(
struct _tagged_arr s))Cyc_badarg)(({const char*_tmp11="printf expected int";
_tag_arr(_tmp11,sizeof(char),_get_zero_arr_size(_tmp11,20));}));_LL17:;}static
unsigned int Cyc_va_arg_uint(struct _tagged_arr ap){void*_tmp12=*((void**)
_check_unknown_subscript(ap,sizeof(void*),0));unsigned int _tmp13;_LL1D: if(*((int*)
_tmp12)!= 1)goto _LL1F;_tmp13=((struct Cyc_Int_pa_struct*)_tmp12)->f1;_LL1E: return(
unsigned int)_tmp13;_LL1F:;_LL20: return((unsigned int(*)(struct _tagged_arr s))Cyc_badarg)(({
const char*_tmp14="printf expected int";_tag_arr(_tmp14,sizeof(char),
_get_zero_arr_size(_tmp14,20));}));_LL1C:;}static double Cyc_va_arg_double(struct
_tagged_arr ap){void*_tmp15=*((void**)_check_unknown_subscript(ap,sizeof(void*),0));
double _tmp16;long double _tmp17;_LL22: if(*((int*)_tmp15)!= 2)goto _LL24;_tmp16=((
struct Cyc_Double_pa_struct*)_tmp15)->f1;_LL23: return _tmp16;_LL24: if(*((int*)
_tmp15)!= 3)goto _LL26;_tmp17=((struct Cyc_LongDouble_pa_struct*)_tmp15)->f1;_LL25:
return(double)_tmp17;_LL26:;_LL27:(int)_throw(Cyc_badarg((struct _tagged_arr)({
struct Cyc_String_pa_struct _tmp1A;_tmp1A.tag=0;_tmp1A.f1=(struct _tagged_arr)((
struct _tagged_arr)Cyc_parg2string(*((void**)_check_unknown_subscript(ap,sizeof(
void*),0))));{void*_tmp18[1]={& _tmp1A};Cyc_aprintf(({const char*_tmp19="printf expected double but found %s";
_tag_arr(_tmp19,sizeof(char),_get_zero_arr_size(_tmp19,36));}),_tag_arr(_tmp18,
sizeof(void*),1));}})));_LL21:;}static short*Cyc_va_arg_short_ptr(struct
_tagged_arr ap){void*_tmp1B=*((void**)_check_unknown_subscript(ap,sizeof(void*),0));
short*_tmp1C;_LL29: if(*((int*)_tmp1B)!= 4)goto _LL2B;_tmp1C=((struct Cyc_ShortPtr_pa_struct*)
_tmp1B)->f1;_LL2A: return _tmp1C;_LL2B:;_LL2C:(int)_throw(Cyc_badarg(({const char*
_tmp1D="printf expected short pointer";_tag_arr(_tmp1D,sizeof(char),
_get_zero_arr_size(_tmp1D,30));})));_LL28:;}static unsigned int*Cyc_va_arg_int_ptr(
struct _tagged_arr ap){void*_tmp1E=*((void**)_check_unknown_subscript(ap,sizeof(
void*),0));unsigned int*_tmp1F;_LL2E: if(*((int*)_tmp1E)!= 5)goto _LL30;_tmp1F=((
struct Cyc_IntPtr_pa_struct*)_tmp1E)->f1;_LL2F: return _tmp1F;_LL30:;_LL31:(int)
_throw(Cyc_badarg(({const char*_tmp20="printf expected int pointer";_tag_arr(
_tmp20,sizeof(char),_get_zero_arr_size(_tmp20,28));})));_LL2D:;}static struct
_tagged_arr Cyc_va_arg_string(struct _tagged_arr ap){void*_tmp21=*((void**)
_check_unknown_subscript(ap,sizeof(void*),0));struct _tagged_arr _tmp22;_LL33: if(*((
int*)_tmp21)!= 0)goto _LL35;_tmp22=((struct Cyc_String_pa_struct*)_tmp21)->f1;
_LL34: return _tmp22;_LL35:;_LL36:(int)_throw(Cyc_badarg(({const char*_tmp23="printf expected string";
_tag_arr(_tmp23,sizeof(char),_get_zero_arr_size(_tmp23,23));})));_LL32:;}int Cyc___cvt_double(
double number,int prec,int flags,int*signp,int fmtch,struct _tagged_arr startp,struct
_tagged_arr endp);enum Cyc_BASE{Cyc_OCT  = 0,Cyc_DEC  = 1,Cyc_HEX  = 2};static int Cyc__IO_sputn(
int(*ioputc)(int,void*),void*ioputc_env,struct _tagged_arr s,int howmany){int i=0;
while(i < howmany){if(_get_arr_size(s,sizeof(char))== 0?1: ioputc((int)*((const
char*)_check_unknown_subscript(s,sizeof(char),0)),ioputc_env)== - 1)return i;
_tagged_arr_inplace_plus_post(& s,sizeof(char),1);i ++;}return i;}static int Cyc__IO_nzsputn(
int(*ioputc)(int,void*),void*ioputc_env,struct _tagged_arr s,int howmany){int i=0;
while(i < howmany){char c;if((_get_arr_size(s,sizeof(char))== 0?1:(c=*((const char*)
_check_unknown_subscript(s,sizeof(char),0)))== 0)?1: ioputc((int)c,ioputc_env)== 
- 1)return i;_tagged_arr_inplace_plus_post(& s,sizeof(char),1);i ++;}return i;}static
int Cyc__IO_padn(int(*ioputc)(int,void*),void*ioputc_env,char c,int howmany){int i=0;
while(i < howmany){if(ioputc((int)c,ioputc_env)== - 1)return i;i ++;}return i;}static
struct _tagged_arr Cyc_my_memchr(struct _tagged_arr s,char c,int n){int sz=(int)
_get_arr_size(s,sizeof(char));n=n < sz?n: sz;for(0;n != 0;(n --,
_tagged_arr_inplace_plus_post(& s,sizeof(char),1))){if(*((const char*)
_check_unknown_subscript(s,sizeof(char),0))== c)return s;}return(struct
_tagged_arr)_tag_arr(0,0,0);}static struct _tagged_arr Cyc_my_nzmemchr(struct
_tagged_arr s,char c,int n){int sz=(int)_get_arr_size(s,sizeof(char));n=n < sz?n: sz;
for(0;n != 0;(n --,_tagged_arr_inplace_plus_post(& s,sizeof(char),1))){if(*((const
char*)_check_unknown_subscript(s,sizeof(char),0))== c)return s;}return(struct
_tagged_arr)_tag_arr(0,0,0);}static unsigned int Cyc_my_strlen(struct _tagged_arr s){
unsigned int sz=_get_arr_size(s,sizeof(char));unsigned int i=0;while(i < sz?((const
char*)s.curr)[(int)i]!= 0: 0){i ++;}return i;}int Cyc__IO_vfprintf(int(*ioputc)(int,
void*),void*ioputc_env,struct _tagged_arr fmt0,struct _tagged_arr ap){struct
_tagged_arr fmt;register int ch;register int n;struct _tagged_arr cp=_tag_arr(0,0,0);
struct _tagged_arr cp2=(struct _tagged_arr)_tag_arr(0,0,0);struct _tagged_arr cp3=(
struct _tagged_arr)_tag_arr(0,0,0);int which_cp;struct _tagged_arr fmark;register int
flags;int ret;int width;int prec;char sign;char sign_string[2]={'\000','\000'};int
softsign=0;double _double;int fpprec;unsigned int _ulong;int dprec;int dpad;int
fieldsz;int size=0;char buf[349];{unsigned int _tmp44=348;unsigned int i;for(i=0;i < 
_tmp44;i ++){buf[i]='\000';}buf[_tmp44]=(char)0;}{char ox[2]={'\000','\000'};enum 
Cyc_BASE base;fmt=fmt0;ret=0;for(0;1;0){fmark=fmt;{unsigned int fmt_sz=
_get_arr_size(fmt,sizeof(char));for(n=0;(n < fmt_sz?(ch=(int)((const char*)fmt.curr)[
n])!= '\000': 0)?ch != '%': 0;n ++){;}fmt=_tagged_arr_plus(fmt,sizeof(char),n);if((n=(
fmt.curr - fmark.curr)/ sizeof(char))!= 0){do{if(Cyc__IO_sputn(ioputc,ioputc_env,
fmark,n)!= n)goto error;}while(0);ret +=n;}if(ch == '\000')goto done;
_tagged_arr_inplace_plus_post(& fmt,sizeof(char),1);flags=0;dprec=0;fpprec=0;
width=0;prec=- 1;sign='\000';rflag: ch=(int)*((const char*)_check_unknown_subscript(
_tagged_arr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0));reswitch:
which_cp=0;switch(ch){case ' ': _LL37: if(!((int)sign))sign=' ';goto rflag;case '#':
_LL38: flags |=8;goto rflag;case '*': _LL39: width=Cyc_va_arg_int(ap);
_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);if(width >= 0)goto rflag;width=
- width;goto _LL3A;case '-': _LL3A: flags |=16;flags &=~ 32;goto rflag;case '+': _LL3B: sign='+';
goto rflag;case '.': _LL3C: if((ch=(int)*((const char*)_check_unknown_subscript(
_tagged_arr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0)))== '*'){n=
Cyc_va_arg_int(ap);_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);prec=n < 0?
- 1: n;goto rflag;}n=0;while((unsigned int)(ch - '0')<= 9){n=10 * n + (ch - '0');ch=(
int)*((const char*)_check_unknown_subscript(_tagged_arr_inplace_plus_post(& fmt,
sizeof(char),1),sizeof(char),0));}prec=n < 0?- 1: n;goto reswitch;case '0': _LL3D: if(!(
flags & 16))flags |=32;goto rflag;case '1': _LL3E: goto _LL3F;case '2': _LL3F: goto _LL40;
case '3': _LL40: goto _LL41;case '4': _LL41: goto _LL42;case '5': _LL42: goto _LL43;case '6':
_LL43: goto _LL44;case '7': _LL44: goto _LL45;case '8': _LL45: goto _LL46;case '9': _LL46: n=
0;do{n=10 * n + (ch - '0');ch=(int)*((const char*)_check_unknown_subscript(
_tagged_arr_inplace_plus_post(& fmt,sizeof(char),1),sizeof(char),0));}while((
unsigned int)(ch - '0')<= 9);width=n;goto reswitch;case 'L': _LL47: flags |=2;goto
rflag;case 'h': _LL48: flags |=4;goto rflag;case 'l': _LL49: flags |=1;goto rflag;case 'c':
_LL4A: cp=({char*_tmp24=buf;_tag_arr(_tmp24,sizeof(char),_get_zero_arr_size(
_tmp24,349));});({struct _tagged_arr _tmp25=cp;char _tmp26=*((char*)
_check_unknown_subscript(_tmp25,sizeof(char),0));char _tmp27=(char)Cyc_va_arg_int(
ap);if(_get_arr_size(_tmp25,sizeof(char))== 1?_tmp26 == '\000'?_tmp27 != '\000': 0:
0)_throw_arraybounds();*((char*)_tmp25.curr)=_tmp27;});
_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);size=1;sign='\000';break;case
'D': _LL4B: flags |=1;goto _LL4C;case 'd': _LL4C: goto _LL4D;case 'i': _LL4D: _ulong=(
unsigned int)(flags & 1?Cyc_va_arg_long(ap):(flags & 4?(int)((short)Cyc_va_arg_int(
ap)):(int)Cyc_va_arg_int(ap)));_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);
if((int)_ulong < 0){_ulong=- _ulong;sign='-';}base=Cyc_DEC;goto number;case 'n':
_LL4E: if(flags & 1)*Cyc_va_arg_int_ptr(ap)=(unsigned int)ret;else{if(flags & 4)*
Cyc_va_arg_short_ptr(ap)=(short)ret;else{*Cyc_va_arg_int_ptr(ap)=(unsigned int)
ret;}}_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);continue;case 'O': _LL4F:
flags |=1;goto _LL50;case 'o': _LL50: _ulong=flags & 1?Cyc_va_arg_ulong(ap):(flags & 4?(
unsigned int)((unsigned short)Cyc_va_arg_int(ap)):(unsigned int)Cyc_va_arg_uint(
ap));_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);base=Cyc_OCT;goto nosign;
case 'p': _LL51: _ulong=(unsigned int)Cyc_va_arg_long(ap);
_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);base=Cyc_HEX;flags |=64;ch=(
int)'x';goto nosign;case 's': _LL52: {struct _tagged_arr b=Cyc_va_arg_string(ap);
_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);which_cp=3;cp3=b;if(prec >= 0){
struct _tagged_arr p=Cyc_my_nzmemchr(cp3,'\000',prec);if(p.curr != ((struct
_tagged_arr)_tag_arr(0,0,0)).curr){size=(p.curr - cp3.curr)/ sizeof(char);if(size
> prec)size=prec;}else{size=prec;}}else{size=(int)Cyc_my_strlen(cp3);}sign='\000';
break;}case 'U': _LL53: flags |=1;goto _LL54;case 'u': _LL54: _ulong=flags & 1?Cyc_va_arg_ulong(
ap):(flags & 4?(unsigned int)((unsigned short)Cyc_va_arg_int(ap)):(unsigned int)
Cyc_va_arg_uint(ap));_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);base=Cyc_DEC;
goto nosign;case 'X': _LL55: goto _LL56;case 'x': _LL56: _ulong=flags & 1?Cyc_va_arg_ulong(
ap):(flags & 4?(unsigned int)((unsigned short)Cyc_va_arg_int(ap)):(unsigned int)
Cyc_va_arg_uint(ap));_tagged_arr_inplace_plus_post(& ap,sizeof(void*),1);base=Cyc_HEX;
if(flags & 8?_ulong != 0: 0)flags |=64;nosign: sign='\000';number: if((dprec=prec)>= 0)
flags &=~ 32;cp=_tagged_arr_plus(({char*_tmp28=buf;_tag_arr(_tmp28,sizeof(char),
_get_zero_arr_size(_tmp28,349));}),sizeof(char),(308 + 39)+ 1);if(_ulong != 0?1:
prec != 0){struct _tagged_arr xdigs;switch(base){case Cyc_OCT: _LL58: do{({struct
_tagged_arr _tmp29=_tagged_arr_inplace_plus(& cp,sizeof(char),-1);char _tmp2A=*((
char*)_check_unknown_subscript(_tmp29,sizeof(char),0));char _tmp2B=(char)((_ulong
& 7)+ '0');if(_get_arr_size(_tmp29,sizeof(char))== 1?_tmp2A == '\000'?_tmp2B != '\000':
0: 0)_throw_arraybounds();*((char*)_tmp29.curr)=_tmp2B;});_ulong >>=3;}while((int)
_ulong);if(flags & 8?*((char*)_check_unknown_subscript(cp,sizeof(char),0))!= '0':
0)({struct _tagged_arr _tmp2C=_tagged_arr_inplace_plus(& cp,sizeof(char),-1);char
_tmp2D=*((char*)_check_unknown_subscript(_tmp2C,sizeof(char),0));char _tmp2E='0';
if(_get_arr_size(_tmp2C,sizeof(char))== 1?_tmp2D == '\000'?_tmp2E != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp2C.curr)=_tmp2E;});break;case Cyc_DEC: _LL59:
while(_ulong >= 10){({struct _tagged_arr _tmp2F=_tagged_arr_inplace_plus(& cp,
sizeof(char),-1);char _tmp30=*((char*)_check_unknown_subscript(_tmp2F,sizeof(char),
0));char _tmp31=(char)(_ulong % 10 + '0');if(_get_arr_size(_tmp2F,sizeof(char))== 1?
_tmp30 == '\000'?_tmp31 != '\000': 0: 0)_throw_arraybounds();*((char*)_tmp2F.curr)=
_tmp31;});_ulong /=10;}({struct _tagged_arr _tmp32=_tagged_arr_inplace_plus(& cp,
sizeof(char),-1);char _tmp33=*((char*)_check_unknown_subscript(_tmp32,sizeof(char),
0));char _tmp34=(char)(_ulong + '0');if(_get_arr_size(_tmp32,sizeof(char))== 1?
_tmp33 == '\000'?_tmp34 != '\000': 0: 0)_throw_arraybounds();*((char*)_tmp32.curr)=
_tmp34;});break;case Cyc_HEX: _LL5A: if(ch == 'X')xdigs=({const char*_tmp35="0123456789ABCDEF";
_tag_arr(_tmp35,sizeof(char),_get_zero_arr_size(_tmp35,17));});else{xdigs=({
const char*_tmp36="0123456789abcdef";_tag_arr(_tmp36,sizeof(char),
_get_zero_arr_size(_tmp36,17));});}do{({struct _tagged_arr _tmp37=
_tagged_arr_inplace_plus(& cp,sizeof(char),-1);char _tmp38=*((char*)
_check_unknown_subscript(_tmp37,sizeof(char),0));char _tmp39=*((const char*)
_check_unknown_subscript(xdigs,sizeof(char),(int)(_ulong & 15)));if(_get_arr_size(
_tmp37,sizeof(char))== 1?_tmp38 == '\000'?_tmp39 != '\000': 0: 0)_throw_arraybounds();*((
char*)_tmp37.curr)=_tmp39;});_ulong >>=4;}while((int)_ulong);break;}}size=((
_tagged_arr_plus(({char*_tmp3A=buf;_tag_arr(_tmp3A,sizeof(char),
_get_zero_arr_size(_tmp3A,349));}),sizeof(char),(308 + 39)+ 1)).curr - cp.curr)/ 
sizeof(char);skipsize: break;default: _LL57: if(ch == '\000')goto done;cp=({char*
_tmp3B=buf;_tag_arr(_tmp3B,sizeof(char),_get_zero_arr_size(_tmp3B,349));});({
struct _tagged_arr _tmp3C=cp;char _tmp3D=*((char*)_check_unknown_subscript(_tmp3C,
sizeof(char),0));char _tmp3E=(char)ch;if(_get_arr_size(_tmp3C,sizeof(char))== 1?
_tmp3D == '\000'?_tmp3E != '\000': 0: 0)_throw_arraybounds();*((char*)_tmp3C.curr)=
_tmp3E;});size=1;sign='\000';break;}fieldsz=size + fpprec;dpad=dprec - size;if(
dpad < 0)dpad=0;if((int)sign)fieldsz ++;else{if(flags & 64)fieldsz +=2;}fieldsz +=
dpad;if((flags & (16 | 32))== 0){if(Cyc__IO_padn(ioputc,ioputc_env,' ',width - 
fieldsz)< width - fieldsz)goto error;}if((int)sign){({struct _tagged_arr _tmp40=
_tagged_arr_plus(({char*_tmp3F=sign_string;_tag_arr(_tmp3F,sizeof(char),
_get_zero_arr_size(_tmp3F,2));}),sizeof(char),0);char _tmp41=*((char*)
_check_unknown_subscript(_tmp40,sizeof(char),0));char _tmp42=sign;if(
_get_arr_size(_tmp40,sizeof(char))== 1?_tmp41 == '\000'?_tmp42 != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp40.curr)=_tmp42;});do{if(Cyc__IO_sputn(ioputc,
ioputc_env,(struct _tagged_arr)({char*_tmp43=sign_string;_tag_arr(_tmp43,sizeof(
char),_get_zero_arr_size(_tmp43,2));}),1)!= 1)goto error;}while(0);}else{if(flags
& 64){ox[0]='0';ox[1]=(char)ch;do{if(Cyc__IO_nzsputn(ioputc,ioputc_env,_tag_arr(
ox,sizeof(char),2),2)!= 2)goto error;}while(0);}}if((flags & (16 | 32))== 32){if(
Cyc__IO_padn(ioputc,ioputc_env,'0',width - fieldsz)< width - fieldsz)goto error;}
if(Cyc__IO_padn(ioputc,ioputc_env,'0',dpad)< dpad)goto error;if(which_cp == 0)do{
if(Cyc__IO_sputn(ioputc,ioputc_env,(struct _tagged_arr)cp,size)!= size)goto error;}while(
0);else{if(which_cp == 2)do{if(Cyc__IO_sputn(ioputc,ioputc_env,cp2,size)!= size)
goto error;}while(0);else{if(which_cp == 3)do{if(Cyc__IO_nzsputn(ioputc,ioputc_env,
cp3,size)!= size)goto error;}while(0);}}if(Cyc__IO_padn(ioputc,ioputc_env,'0',
fpprec)< fpprec)goto error;if(flags & 16){if(Cyc__IO_padn(ioputc,ioputc_env,' ',
width - fieldsz)< width - fieldsz)goto error;}ret +=width > fieldsz?width: fieldsz;}}
done: return ret;error: return - 1;}}static struct _tagged_arr Cyc_exponent(struct
_tagged_arr p,int exp,int fmtch){struct _tagged_arr t;char expbuf[309];{unsigned int
_tmp5F=308;unsigned int i;for(i=0;i < _tmp5F;i ++){expbuf[i]='0';}expbuf[_tmp5F]=(
char)0;}({struct _tagged_arr _tmp45=_tagged_arr_inplace_plus_post(& p,sizeof(char),
1);char _tmp46=*((char*)_check_unknown_subscript(_tmp45,sizeof(char),0));char
_tmp47=(char)fmtch;if(_get_arr_size(_tmp45,sizeof(char))== 1?_tmp46 == '\000'?
_tmp47 != '\000': 0: 0)_throw_arraybounds();*((char*)_tmp45.curr)=_tmp47;});if(exp < 
0){exp=- exp;({struct _tagged_arr _tmp48=_tagged_arr_inplace_plus_post(& p,sizeof(
char),1);char _tmp49=*((char*)_check_unknown_subscript(_tmp48,sizeof(char),0));
char _tmp4A='-';if(_get_arr_size(_tmp48,sizeof(char))== 1?_tmp49 == '\000'?_tmp4A
!= '\000': 0: 0)_throw_arraybounds();*((char*)_tmp48.curr)=_tmp4A;});}else{({
struct _tagged_arr _tmp4B=_tagged_arr_inplace_plus_post(& p,sizeof(char),1);char
_tmp4C=*((char*)_check_unknown_subscript(_tmp4B,sizeof(char),0));char _tmp4D='+';
if(_get_arr_size(_tmp4B,sizeof(char))== 1?_tmp4C == '\000'?_tmp4D != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp4B.curr)=_tmp4D;});}t=_tagged_arr_plus(({char*
_tmp4E=expbuf;_tag_arr(_tmp4E,sizeof(char),_get_zero_arr_size(_tmp4E,309));}),
sizeof(char),308);if(exp > 9){do{({struct _tagged_arr _tmp4F=
_tagged_arr_inplace_plus(& t,sizeof(char),-1);char _tmp50=*((char*)
_check_unknown_subscript(_tmp4F,sizeof(char),0));char _tmp51=(char)(exp % 10 + '0');
if(_get_arr_size(_tmp4F,sizeof(char))== 1?_tmp50 == '\000'?_tmp51 != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp4F.curr)=_tmp51;});}while((exp /=10)> 9);({
struct _tagged_arr _tmp52=_tagged_arr_inplace_plus(& t,sizeof(char),-1);char _tmp53=*((
char*)_check_unknown_subscript(_tmp52,sizeof(char),0));char _tmp54=(char)(exp + '0');
if(_get_arr_size(_tmp52,sizeof(char))== 1?_tmp53 == '\000'?_tmp54 != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp52.curr)=_tmp54;});for(0;t.curr < (
_tagged_arr_plus(({char*_tmp55=expbuf;_tag_arr(_tmp55,sizeof(char),
_get_zero_arr_size(_tmp55,309));}),sizeof(char),308)).curr;({struct _tagged_arr
_tmp56=_tagged_arr_inplace_plus_post(& p,sizeof(char),1);char _tmp57=*((char*)
_check_unknown_subscript(_tmp56,sizeof(char),0));char _tmp58=*((char*)
_check_unknown_subscript(_tagged_arr_inplace_plus_post(& t,sizeof(char),1),
sizeof(char),0));if(_get_arr_size(_tmp56,sizeof(char))== 1?_tmp57 == '\000'?
_tmp58 != '\000': 0: 0)_throw_arraybounds();*((char*)_tmp56.curr)=_tmp58;})){;}}
else{({struct _tagged_arr _tmp59=_tagged_arr_inplace_plus_post(& p,sizeof(char),1);
char _tmp5A=*((char*)_check_unknown_subscript(_tmp59,sizeof(char),0));char _tmp5B='0';
if(_get_arr_size(_tmp59,sizeof(char))== 1?_tmp5A == '\000'?_tmp5B != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp59.curr)=_tmp5B;});({struct _tagged_arr _tmp5C=
_tagged_arr_inplace_plus_post(& p,sizeof(char),1);char _tmp5D=*((char*)
_check_unknown_subscript(_tmp5C,sizeof(char),0));char _tmp5E=(char)(exp + '0');if(
_get_arr_size(_tmp5C,sizeof(char))== 1?_tmp5D == '\000'?_tmp5E != '\000': 0: 0)
_throw_arraybounds();*((char*)_tmp5C.curr)=_tmp5E;});}return p;}int Cyc_vfprintf(
struct Cyc___cycFILE*f,struct _tagged_arr fmt,struct _tagged_arr ap){int ans;ans=((int(*)(
int(*ioputc)(int,struct Cyc___cycFILE*),struct Cyc___cycFILE*ioputc_env,struct
_tagged_arr fmt0,struct _tagged_arr ap))Cyc__IO_vfprintf)(Cyc_putc,f,fmt,ap);return
ans;}int Cyc_fprintf(struct Cyc___cycFILE*f,struct _tagged_arr fmt,struct _tagged_arr
ap){return Cyc_vfprintf(f,fmt,ap);}int Cyc_vprintf(struct _tagged_arr fmt,struct
_tagged_arr ap){int ans;ans=((int(*)(int(*ioputc)(int,struct Cyc___cycFILE*),struct
Cyc___cycFILE*ioputc_env,struct _tagged_arr fmt0,struct _tagged_arr ap))Cyc__IO_vfprintf)(
Cyc_putc,Cyc_stdout,fmt,ap);return ans;}int Cyc_printf(struct _tagged_arr fmt,struct
_tagged_arr ap){int ans;ans=Cyc_vprintf(fmt,ap);return ans;}struct _tuple0{struct
_tagged_arr*f1;unsigned int*f2;};static int Cyc_putc_string(int c,struct _tuple0*
sptr_n){struct _tuple0 _tmp61;struct _tagged_arr*_tmp62;unsigned int*_tmp63;struct
_tuple0*_tmp60=sptr_n;_tmp61=*_tmp60;_tmp62=_tmp61.f1;_tmp63=_tmp61.f2;{struct
_tagged_arr s=*_tmp62;unsigned int n=*_tmp63;if(_get_arr_size(s,sizeof(char))== 0?
1: n == 0)return - 1;*((char*)_check_unknown_subscript(s,sizeof(char),0))=(char)c;*
_tmp62=_tagged_arr_plus(s,sizeof(char),1);*_tmp63=n - 1;return 1;}}int Cyc_vsnprintf(
struct _tagged_arr s,unsigned int n,struct _tagged_arr fmt,struct _tagged_arr ap){int
ans;struct _tagged_arr _tmp64=s;unsigned int _tmp65=n;struct _tuple0 _tmp66=({struct
_tuple0 _tmp67;_tmp67.f1=& _tmp64;_tmp67.f2=& _tmp65;_tmp67;});ans=((int(*)(int(*
ioputc)(int,struct _tuple0*),struct _tuple0*ioputc_env,struct _tagged_arr fmt0,
struct _tagged_arr ap))Cyc__IO_vfprintf)(Cyc_putc_string,& _tmp66,fmt,ap);if(0 <= 
ans)*((char*)_check_unknown_subscript(s,sizeof(char),ans))='\000';return ans;}int
Cyc_snprintf(struct _tagged_arr s,unsigned int n,struct _tagged_arr fmt,struct
_tagged_arr ap){return Cyc_vsnprintf(s,n,fmt,ap);}int Cyc_vsprintf(struct
_tagged_arr s,struct _tagged_arr fmt,struct _tagged_arr ap){return Cyc_vsnprintf(s,
_get_arr_size(s,sizeof(char)),fmt,ap);}int Cyc_sprintf(struct _tagged_arr s,struct
_tagged_arr fmt,struct _tagged_arr ap){return Cyc_vsnprintf(s,_get_arr_size(s,
sizeof(char)),fmt,ap);}static int Cyc_putc_void(int c,int dummy){return 1;}struct
_tagged_arr Cyc_vrprintf(struct _RegionHandle*r1,struct _tagged_arr fmt,struct
_tagged_arr ap){int size=((int(*)(int(*ioputc)(int,int),int ioputc_env,struct
_tagged_arr fmt0,struct _tagged_arr ap))Cyc__IO_vfprintf)(Cyc_putc_void,0,fmt,ap)+ 
1;struct _tagged_arr s=({unsigned int _tmp68=(unsigned int)size;char*_tmp69=(char*)
_region_malloc(r1,_check_times(sizeof(char),_tmp68 + 1));struct _tagged_arr _tmp6B=
_tag_arr(_tmp69,sizeof(char),_tmp68 + 1);{unsigned int _tmp6A=_tmp68;unsigned int i;
for(i=0;i < _tmp6A;i ++){_tmp69[i]='\000';}_tmp69[_tmp6A]=(char)0;}_tmp6B;});Cyc_vsprintf(
_tagged_ptr_decrease_size(s,sizeof(char),1),fmt,ap);return s;}struct _tagged_arr
Cyc_rprintf(struct _RegionHandle*r1,struct _tagged_arr fmt,struct _tagged_arr ap){
return Cyc_vrprintf(r1,fmt,ap);}struct _tagged_arr Cyc_aprintf(struct _tagged_arr fmt,
struct _tagged_arr ap){return Cyc_vrprintf(Cyc_Core_heap_region,fmt,ap);}
